package fitconnect.api;

import fitconnect.api.auth.OAuthToken;
import fitconnect.api.data.Data;
import fitconnect.api.validation.ValidationResult;

import java.io.ByteArrayInputStream;

/**
 * The technical system that accepts submissions on the administration side.
 */
public interface Subscriber {

    /**
     * Authenticates the sender against the FitCo API and retrieves an authentication token
     * @param clientId -
     * @param clientSecret -
     * @return
     */
    OAuthToken retrieveAuthenticationToken(final String clientId, final String clientSecret);

    /**
     * Decrypts the subscribed data with JWE.
     *
     * @param encryptedData - encrypted JSON or XML data
     * @return decrypted JSON or XML data
     */
    Data decryptData(final Data encryptedData);

    /**
     * Encrypts the submission attachment with JWE.
     *
     * @param unencryptedAttachmentBinary - the unencrypted binary data of the attachment
     * @return the encrypted binary of the attachment
     */
    ByteArrayInputStream decryptAttachment(final ByteArrayInputStream unencryptedAttachmentBinary);

    /**
     * Validates metadata against a given json-schema
     *
     * @param attachment - the unencrypted binary data of the attachment
     * @param jsonSchema - the schema the metadata is validated with
     * @return A {@link ValidationResult} that informs about the validation and potential errors
     */
    ValidationResult validateMetadata(byte[] attachment, String jsonSchema);


}
